"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var enzyme_1 = require("enzyme");
var react_1 = __importDefault(require("react"));
var sinon_1 = __importDefault(require("sinon"));
var __1 = require("../../");
var examples_1 = require("./examples");
var promises_1 = require("../../utils/promises");
var schema = {
    type: 'object',
    properties: {
        Name: {
            title: 'Pokemon Name',
            type: 'string',
        },
    },
    additionalProperties: true,
};
describe('Form component', function () {
    it('should still render if an unknown format is used', function () {
        var uuidSchema = {
            type: 'object',
            properties: {
                id: {
                    type: 'string',
                    format: 'uuid',
                },
            },
        };
        (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
            react_1.default.createElement(__1.Form, { schema: uuidSchema })));
    });
    describe('hideSubmitButton property', function () {
        it('should render a submit button if the property is not specified', function () {
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema })));
            expect(component.find('button')).toHaveLength(1);
            component.unmount();
        });
        it('should render a submit button if the property is falsey', function () {
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, hideSubmitButton: false })));
            expect(component.find('button')).toHaveLength(1);
            component.unmount();
        });
        it('should not render a submit button if the property is truthey', function () {
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, hideSubmitButton: true })));
            expect(component.find('button')).toHaveLength(0);
            component.unmount();
        });
    });
    describe('submitButtonText property', function () {
        it('should change the text value of the submit button', function () {
            var submitText = 'Click me to submit this form';
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, submitButtonText: submitText })));
            expect(component.find('button').text()).toEqual(submitText);
            component.unmount();
        });
        it('should accept a JSX element as a value', function () {
            var submitText = 'Click me to submit this form';
            var submitElement = react_1.default.createElement("span", null, submitText);
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, submitButtonText: submitElement })));
            expect(component.find('button').text()).toEqual(submitText);
            component.unmount();
        });
    });
    describe('submitButtonProps property', function () {
        it('should set submit button props', function () {
            var submitButtonProps = {
                className: 'custom-button-class',
            };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, submitButtonProps: submitButtonProps })));
            expect(component.find('button').hasClass(submitButtonProps.className)).toEqual(true);
            component.unmount();
        });
    });
    describe('onFormSubmit property', function () {
        it('should be called when the submit button is clicked', function () {
            var value = 'Squirtle';
            var callback = sinon_1.default.spy();
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, onFormSubmit: callback })));
            var input = component.find('input');
            input.simulate('change', { target: { value: value } });
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                component.find('form').simulate('submit');
                expect(callback.callCount).toEqual(1);
                expect(callback.getCall(0).args[0].formData).toEqual({ Name: value });
                component.unmount();
            });
        });
    });
    describe('onFormChange property', function () {
        it('should be called when an input field is changed', function () {
            var value = 'Squirtle';
            var callback = sinon_1.default.spy();
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, onFormChange: callback })));
            var input = component.find('input');
            input.simulate('change', { target: { value: value } });
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                expect(callback.called).toEqual(true);
                expect(callback.lastCall.args[0].formData).toEqual({ Name: value });
                component.unmount();
            });
        });
    });
    describe('value property', function () {
        it('should set the value of the relevant input field', function () {
            var value = { Name: 'Squirtle' };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, value: value })));
            var input = component.find('input');
            expect(input.props().value).toEqual('Squirtle');
            component.unmount();
        });
        it('should set the value if the value is changed after the component mounts', function () {
            var value = { Name: 'Squirtle' };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(function (props) { return (react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, __assign({}, props)))); }, { schema: schema }));
            component.setProps({ value: value });
            component.update();
            var input = component.find('input');
            expect(input.props().value).toEqual('Squirtle');
            component.unmount();
        });
    });
    describe('enum field', function () {
        var schema = {
            type: 'object',
            properties: {
                testfield: {
                    title: 'Test',
                    description: 'Test field description',
                    enum: ['foo', 'bar'],
                    enumNames: ['Foo', 'Bar'],
                },
            },
        };
        it('should display the enum names', function () {
            var callback = sinon_1.default.spy();
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, onFormChange: callback })));
            component.find('button').first().simulate('click');
            expect(component
                .find('button[role="menuitem"]')
                .first()
                .find('span')
                .first()
                .text()).toBe('Foo');
            expect(component
                .find('button[role="menuitem"]')
                .at(1)
                .find('span')
                .first()
                .text()).toBe('Bar');
        });
        it('should call form change when an option is selected', function () {
            var callback = sinon_1.default.spy();
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, onFormChange: callback })));
            component.find('button').first().simulate('click');
            component
                .find('button[role="menuitem"]')
                .first()
                .find('span')
                .first()
                .simulate('click');
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                var callArg = callback.lastCall.args[0];
                expect(callArg.formData).toEqual({ testfield: 'foo' });
            });
        });
    });
    describe('top-level array fields', function () {
        var arraySchema = {
            items: {
                type: 'string',
            },
            type: 'array',
        };
        var callback = sinon_1.default.spy();
        it('should not crash on render', function () {
            (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: arraySchema, onFormChange: callback })));
        });
    });
    describe('array fields', function () {
        var arraySchema = {
            type: 'object',
            properties: {
                Array: {
                    type: 'array',
                    items: {
                        type: 'string',
                    },
                },
            },
        };
        var callback = sinon_1.default.spy();
        var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
            react_1.default.createElement(__1.Form, { schema: arraySchema, onFormChange: callback })));
        var value1 = 'foobarbaz';
        var value2 = 'bazbarfoo';
        it('should be able to add an item', function () {
            var value = 'foobarbaz';
            component
                .find('.rendition-form-array-item__add-item')
                .first()
                .simulate('click');
            var input = component.find('input').first();
            input.simulate('change', {
                target: {
                    value: value1,
                },
            });
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                var callArg = callback.lastCall.args[0];
                expect(callArg.formData).toEqual({
                    Array: [value],
                });
            });
        });
        it('should be able to add multiple items', function () {
            component
                .find('.rendition-form-array-item__add-item')
                .first()
                .simulate('click');
            var input = component.find('input').last();
            input.simulate('change', {
                target: {
                    value: value2,
                },
            });
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                var callArg = callback.lastCall.args[0];
                expect(callArg.formData).toEqual({
                    Array: [value1, value2],
                });
            });
        });
        it('should be able to move an item up', function () {
            component
                .find('.rendition-form-array-item__move-up')
                .last()
                .simulate('click');
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                var callArg = callback.lastCall.args[0];
                expect(callArg.formData).toEqual({
                    Array: [value2, value1],
                });
            });
        });
        it('should be able to move an item down', function () {
            component
                .find('.rendition-form-array-item__move-down')
                .first()
                .simulate('click');
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                var callArg = callback.lastCall.args[0];
                expect(callArg.formData).toEqual({
                    Array: [value1, value2],
                });
            });
        });
        it('should be able to remove an item', function () {
            component
                .find('.rendition-form-array-item__remove-item')
                .last()
                .simulate('click');
            component.update();
            return (0, promises_1.delay)(150).then(function () {
                var callArg = callback.lastCall.args[0];
                expect(callArg.formData).toEqual({
                    Array: [value1],
                });
                component.unmount();
            });
        });
    });
    describe('uiSchema property', function () {
        it('should be able to add warnings using ui:warning', function () {
            var uiSchema = {
                Name: {
                    'ui:warning': 'lorem ipsum dolor sit amet',
                },
            };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { uiSchema: uiSchema, schema: schema })));
            var warnings = component.find(__1.Alert);
            expect(warnings.length).toEqual(1);
            expect(warnings.first().text().trim()).toEqual(uiSchema.Name['ui:warning']);
            component.unmount();
        });
    });
    describe('description keyword', function () {
        it('should support markdown', function () {
            var descriptionSchema = {
                type: 'object',
                properties: {
                    foo: {
                        type: 'string',
                        description: 'Lorem ipsum *dolor* sit amet',
                    },
                },
            };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: descriptionSchema, extraFormats: examples_1.EXTRA_FORMATS })));
            var description = component.find('.rendition-form-description').first();
            expect(description.html()).toMatchSnapshot();
            component.unmount();
        });
    });
    describe('ui:help keyword', function () {
        it('should support markdown', function () {
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { extraFormats: examples_1.EXTRA_FORMATS, schema: {
                        type: 'object',
                        properties: {
                            foo: {
                                type: 'string',
                            },
                        },
                    }, uiSchema: {
                        foo: {
                            'ui:help': 'Lorem ipsum *dolor* sit amet',
                        },
                    } })));
            var description = component.find('.rendition-form-help').first();
            expect(description.html()).toMatchSnapshot();
            component.unmount();
        });
    });
    describe('field ids', function () {
        it('should generate a unique class name for each field', function () {
            var multiFieldSchema = {
                type: 'object',
                properties: {
                    Name: {
                        title: 'Pokemon Name',
                        type: 'string',
                    },
                    Height: {
                        type: 'number',
                    },
                    Weight: {
                        type: 'number',
                    },
                    Description: {
                        type: 'string',
                    },
                },
            };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: multiFieldSchema })));
            var expectedClassNames = Object.keys(multiFieldSchema.properties).map(function (key) {
                return "rendition-form__field--root_".concat(key);
            });
            expectedClassNames.forEach(function (className) {
                expect(component.find(".".concat(className)).hostNodes()).toHaveLength(1);
            });
            component.unmount();
        });
    });
    describe('patternProperties', function () {
        it('should allow you to add new key value pairs', function () {
            var schema = {
                type: 'object',
                patternProperties: {
                    '^[0-9]+$': {
                        title: 'Rule',
                        type: 'string',
                    },
                },
            };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema })));
            var valueInput = component.find('input.rendition-form-pattern-properties__value-field');
            expect(valueInput.length).toEqual(1);
            var keyInput = component
                .find('.rendition-form-pattern-properties__key-field')
                .first();
            expect(keyInput.length).toEqual(1);
        });
        it('should seperate out defined keys', function () {
            var schema = {
                type: 'object',
                properties: {
                    foo: {
                        type: 'string',
                    },
                },
                patternProperties: {
                    '^[0-9]+$': {
                        title: 'Rule',
                        type: 'string',
                    },
                },
            };
            var component = (0, enzyme_1.mount)(react_1.default.createElement(__1.Provider, null,
                react_1.default.createElement(__1.Form, { schema: schema, value: {
                        foo: 'bar',
                    } })));
            var keyInput = component
                .find('.rendition-form-pattern-properties__key-field')
                .first();
            expect(keyInput.props().value).toEqual('');
            var fooInput = component.find('input#root_foo');
            expect(fooInput.props().value).toEqual('bar');
        });
    });
});
