import { __assign, __spreadArray } from "tslib";
import Ajv from 'ajv';
import ajvKeywords from 'ajv-keywords';
import { getDataModel, isDateTimeFormat } from '../DataTypes';
import { findInObject, isJson, randomString } from '../../utils';
import pickBy from 'lodash/pickBy';
import { getJsonDescription } from '../DataTypes/utils';
import pick from 'lodash/pick';
import get from 'lodash/get';
var ajv = new Ajv();
ajvKeywords(ajv, ['regexp', 'formatMaximum', 'formatMinimum']);
export var FULL_TEXT_SLUG = 'full_text_search';
export var getSignatures = function (filter) {
    var _a;
    var signatures = filter.anyOf.map(function (f) {
        return parseDescription(f);
    });
    return ((_a = signatures.filter(function (s) { return s !== null; })) !== null && _a !== void 0 ? _a : []).map(function (s) { return (__assign(__assign({}, s), { key: !!s.refScheme ? "".concat(s.field, "__").concat(s.refScheme) : s.field })); });
};
var getSignatureLabel = function (schema, value) {
    if (Array.isArray(schema.oneOf)) {
        var oneOf = schema.oneOf.find(function (oneOf) { return oneOf && typeof oneOf === 'object' && oneOf.const === value; });
        return typeof oneOf === 'object' ? oneOf.title : undefined;
    }
    return undefined;
};
export var createFilter = function (schema, signatures) {
    var anyOf = signatures.map(function (_a) {
        var field = _a.field, operator = _a.operator, value = _a.value, refScheme = _a.refScheme;
        var subSchema = !!refScheme
            ? generateSchemaFromRefScheme(schema, field, refScheme)
            : __assign({}, schema.properties[field]);
        var label = getSignatureLabel(subSchema, value);
        var model = getDataModel(subSchema);
        if (!model) {
            return {};
        }
        return __assign({ $id: randomString(), title: operator.slug, description: getJsonDescription(subSchema.title, field, operator, value, label, refScheme), type: 'object' }, model.createFilter(field, operator, value, subSchema, refScheme));
    });
    return {
        $id: randomString(),
        anyOf: anyOf,
    };
};
export var createFullTextSearchFilter = function (schema, term) {
    var items = !!schema.properties
        ? Object.entries(schema.properties).reduce(function (itemsAccumulator, entry) {
            var _a;
            var key = entry[0], item = entry[1];
            var model = getDataModel(item);
            if (((_a = item.type) === null || _a === void 0 ? void 0 : _a.includes('boolean')) ||
                isDateTimeFormat(item.format) ||
                !model) {
                return itemsAccumulator;
            }
            itemsAccumulator.push({
                key: key,
                schema: item,
                model: model,
            });
            return itemsAccumulator;
        }, [])
        : [];
    var filter = generateFullTextSearchAjvFilter(term, items);
    return filter;
};
var generateFullTextSearchAjvFilter = function (term, items) {
    var operator = {
        label: 'full text search',
        slug: FULL_TEXT_SLUG,
    };
    return {
        $id: randomString(),
        title: FULL_TEXT_SLUG,
        anyOf: [
            {
                title: FULL_TEXT_SLUG,
                description: getJsonDescription('Any field', 'any', operator, term, undefined, undefined),
                anyOf: items
                    .map(function (_a) {
                    var key = _a.key, schema = _a.schema, model = _a.model;
                    if (!model) {
                        return {};
                    }
                    return model.createFilter(key, operator, term, schema);
                })
                    .filter(function (item) { return Object.keys(item).length > 0; }),
            },
        ],
    };
};
export function filter(filters, collection) {
    ajv.removeSchema(/^.*$/);
    var validators = Array.isArray(filters)
        ? filters.map(function (s) { return ajv.compile(s); })
        : [ajv.compile(filters)];
    if (Array.isArray(collection)) {
        return collection.filter(function (m) { return validators.every(function (v) { return v(m); }); });
    }
    return pickBy(collection, function (m) { return validators.every(function (v) { return v(m); }); });
}
export var parseDescription = function (schemaValue) {
    return schemaValue &&
        typeof schemaValue !== 'boolean' &&
        schemaValue.description &&
        isJson(schemaValue.description)
        ? JSON.parse(schemaValue.description)
        : null;
};
export var parseDescriptionProperty = function (schemaValue, property) {
    var description = parseDescription(schemaValue);
    return description && property in description
        ? description[property]
        : null;
};
export var getOperators = function (schema, field, refScheme) {
    var subSchema = (refScheme
        ? getSubSchemaFromRefScheme(schema.properties[field], refScheme)
        : schema.properties[field]);
    var model = getDataModel(subSchema);
    if (!model) {
        return [];
    }
    return Object.entries(model.operators(subSchema)).map(function (_a) {
        var key = _a[0], value = _a[1];
        return ({
            slug: key,
            label: value,
        });
    });
};
export var getModelFilter = function (schema, field, operator, value) {
    var model = getDataModel(schema);
    if (!model) {
        throw new Error("getModelFilter: model not defied, something is probably wrong with the type declaration in the schema! field: ".concat(field, ", type: ").concat(schema.type));
    }
    var filter = model.createFilter(field, operator, value, schema);
    return filter;
};
export var getSchemaFormat = function (schema) {
    var _a;
    var property = getSubSchemaFromRefScheme(schema);
    var format = (_a = property.format) !== null && _a !== void 0 ? _a : schema.format;
    return format;
};
export var getPropertyScheme = function (schemaValue) {
    var _a;
    var json = parseDescription(schemaValue);
    return json && ('x-foreign-key-scheme' in json || 'x-ref-scheme' in json)
        ? (_a = json['x-foreign-key-scheme']) !== null && _a !== void 0 ? _a : json['x-ref-scheme']
        : null;
};
export var convertRefSchemeToSchemaPath = function (refScheme) {
    return !!refScheme
        ? refScheme
            .split('.')
            .join('.properties.')
            .replace(/\[\d+\]/g, '.items')
        :
            refScheme;
};
export var getSchemaTitle = function (jsonSchema, propertyKey, refScheme) {
    var _a, _b, _c;
    if (!refScheme) {
        return (jsonSchema === null || jsonSchema === void 0 ? void 0 : jsonSchema.title) || propertyKey;
    }
    return ((_c = (_b = (_a = getSubSchemaFromRefScheme(jsonSchema, refScheme)) === null || _a === void 0 ? void 0 : _a.title) !== null && _b !== void 0 ? _b : jsonSchema.title) !== null && _c !== void 0 ? _c : propertyKey);
};
export var generateSchemaFromRefScheme = function (schema, parentProperty, refScheme) {
    var _a, _b, _c, _d;
    var propertySchema = (_b = (_a = schema.properties) === null || _a === void 0 ? void 0 : _a[parentProperty]) !== null && _b !== void 0 ? _b : schema;
    if (!refScheme) {
        return propertySchema;
    }
    var convertedRefScheme = propertySchema.items
        ? "items.properties.".concat(convertRefSchemeToSchemaPath(refScheme))
        : "properties.".concat(convertRefSchemeToSchemaPath(refScheme));
    var typePaths = [];
    var ongoingIncrementalPath = [];
    convertedRefScheme.split('.').forEach(function (key) {
        if (['properties', 'items'].includes(key)) {
            typePaths.push(__spreadArray(__spreadArray([], ongoingIncrementalPath, true), ['type'], false));
        }
        ongoingIncrementalPath.push(key);
    });
    if (ongoingIncrementalPath.length) {
        typePaths.push(ongoingIncrementalPath);
    }
    var referenceSchema = get(propertySchema, convertedRefScheme);
    var referenceSchemaDescription = (_c = parseDescription(referenceSchema)) !== null && _c !== void 0 ? _c : {};
    return __assign(__assign(__assign({}, propertySchema), { description: "".concat(JSON.stringify(Object.assign(referenceSchemaDescription, {
            'x-ref-scheme': [refScheme],
        }))), title: (_d = referenceSchema === null || referenceSchema === void 0 ? void 0 : referenceSchema.title) !== null && _d !== void 0 ? _d : propertySchema.title }), pick(propertySchema, typePaths));
};
export var getSubSchemaFromRefScheme = function (schema, refScheme) {
    var _a;
    var referenceScheme = refScheme || ((_a = getPropertyScheme(schema)) === null || _a === void 0 ? void 0 : _a[0]);
    var convertedRefScheme = convertRefSchemeToSchemaPath(referenceScheme);
    if (!convertedRefScheme) {
        return schema;
    }
    var properties = findInObject(schema, 'properties');
    return get(properties, convertedRefScheme);
};
