import { __assign } from "tslib";
import { mount } from 'enzyme';
import React from 'react';
import { Filters, Provider, SchemaSieve, Select, Tag, DropDownButton, } from '../../';
import { Summary } from '../../components/Filters/Summary';
import { Keyboard } from 'grommet';
var schema = {
    type: 'object',
    properties: {
        Name: {
            title: 'Pokemon Name',
            type: 'string',
        },
        Tag: {
            title: 'Tag',
            type: 'object',
            properties: {
                tag_name: {
                    title: 'Tag name',
                    description: 'key',
                    type: 'string',
                },
                tag_value: {
                    description: 'value',
                    title: 'Tag value',
                    type: 'string',
                },
            },
        },
    },
};
var filter = SchemaSieve.createFilter(schema, [
    {
        title: 'Name',
        field: 'Name',
        operator: { slug: 'is', label: 'is' },
        value: 'Squirtle',
    },
]);
var tagIsFilter = SchemaSieve.createFilter(schema, [
    {
        title: 'Tag',
        field: 'Tag',
        operator: { slug: 'is', label: 'is' },
        value: { tag_name: 'rarity', tag_value: '10' },
    },
]);
var tagIsNotFilter = SchemaSieve.createFilter(schema, [
    {
        title: 'Tag',
        field: 'Tag',
        operator: { slug: 'is_not', label: 'is not' },
        value: { tag_name: 'rarity', tag_value: '10' },
    },
]);
var view = {
    id: 'abcde',
    name: 'Test',
    scope: null,
    filters: [filter],
};
describe('Filters component', function () {
    describe('filters property', function () {
        it('should not render a summary if there are no filters', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema })));
            expect(component.find(Summary)).toHaveLength(0);
            component.unmount();
        });
        it('should render a summary of filters that are provided as props', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [filter] })));
            expect(component.find(Summary)).toHaveLength(1);
            component.unmount();
        });
        it('should render a summary of filters that are added as props after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema }));
            component.setProps({ filters: [filter] });
            component.update();
            expect(component.find(Summary)).toHaveLength(1);
            component.unmount();
        });
        it('should not render a summary of filters if they are removed after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema, filters: [filter] }));
            component.setProps({ filters: null });
            component.update();
            expect(component.find(Summary)).toHaveLength(0);
            component.unmount();
        });
        it('should show the correct label when filtering using "is" operator for both key and value properties of an object', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [tagIsFilter] })));
            expect(component.find(Tag).text()).toBe('Tag is tag_name: rarity, tag_value: 10');
        });
        it('should show the correct label when filtering using "is_not" operator for both key and value properties of an object', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [tagIsNotFilter] })));
            expect(component.find(Tag).text()).toBe('Tag is not tag_name: rarity, tag_value: 10');
        });
        it('should remove buttons labels if compact property is set to true', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { compact: [true, true, true, true], schema: schema })));
            expect(component.find('button').at(0).text()).toBe('');
            expect(component.find('button').at(1).text()).toBe('');
        });
    });
    describe('save views modal', function () {
        it('should not show scopes selector if one or less scopes are passed', function () {
            var view = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [filter], showSaveView: true })));
            view.find('button').at(3).simulate('click');
            expect(view.find(Select)).toHaveLength(0);
            view.unmount();
        });
    });
    describe('views property', function () {
        it('should not render a views menu if there are no views', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema })));
            component.find(DropDownButton).find('button').simulate('click');
            expect(component.html()).toContain("You haven't created any views yet");
            component.unmount();
        });
        it('should render a list of views that are provided as props', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, views: [view] })));
            component.find(DropDownButton).find('button').simulate('click');
            expect(component.find('svg')).toHaveLength(5);
            component.unmount();
        });
        it('should render a list of views that are added as props after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema }));
            component.setProps({ views: [view] });
            component.update();
            component.find(DropDownButton).find('button').simulate('click');
            expect(component.find('svg')).toHaveLength(5);
            component.unmount();
        });
        it('should not render a list of views if they are removed after the component mounts', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Filters, __assign({}, props)))); }, { schema: schema, views: [view] }));
            component.setProps({ views: null });
            component.update();
            component.find(DropDownButton).find('button').simulate('click');
            expect(component.html()).toContain("You haven't created any views yet");
            component.unmount();
        });
        it('should render when the schema contains an unknown type', function () {
            var unknownSchema = {
                type: 'object',
                properties: {
                    test: {
                        type: 'Foo Bar',
                    },
                },
            };
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: unknownSchema })));
            component.unmount();
        });
        it('should clear all filters when `clear all filters` gets clicked', function () {
            var defaultFilters = SchemaSieve.createFilter(schema, [
                {
                    title: 'Name',
                    field: 'Name',
                    operator: { slug: 'contains', label: 'contains' },
                    value: 's',
                },
                {
                    title: 'Name',
                    field: 'Name',
                    operator: { slug: 'contains', label: 'contains' },
                    value: 'q',
                },
            ]);
            var component = mount(React.createElement(Provider, null,
                React.createElement(Filters, { schema: schema, filters: [defaultFilters] })));
            expect(component.find(Summary)).toHaveLength(1);
            component
                .find('input')
                .simulate('change', { target: { value: 'Squirtle' } });
            component.find(Keyboard).simulate('keypress', { key: 'Enter' });
            expect(component.text()).toContain('Clear all');
            component
                .findWhere(function (node) { return node.text() === 'Clear all'; })
                .at(1)
                .simulate('click');
            expect(component.find(Summary)).toHaveLength(0);
            component.unmount();
        });
    });
});
