import { __assign } from "tslib";
import { mount } from 'enzyme';
import first from 'lodash/first';
import get from 'lodash/get';
import last from 'lodash/last';
import map from 'lodash/map';
import noop from 'lodash/noop';
import reverse from 'lodash/reverse';
import React from 'react';
import sinon from 'sinon';
import PokeDex from '../../stories/assets/pokedex';
import { Pager, Provider, Table } from '../../';
import { TableColumnSelectorSizer } from '.';
import { TableBase } from './TableBase';
var columns = [
    {
        field: 'Name',
        sortable: true,
        selected: true,
    },
    {
        field: 'pokedex_number',
        label: 'National Pokedex Number',
        sortable: true,
        selected: true,
        render: function (value) { return React.createElement("code", null, value); },
    },
    {
        field: 'Category',
        sortable: true,
        selected: false,
    },
    {
        field: 'first_seen',
        label: 'First Seen',
        sortable: true,
        selected: true,
    },
];
var getRowClass = function (pokemon) {
    var classNames = ['pokemon'];
    if (pokemon.caught) {
        classNames.push('pokemon--caught');
    }
    if (pokemon.Abilities.length === 1) {
        classNames.push('pokemon--one-dimensional');
    }
    return classNames;
};
describe('Table component', function () {
    describe('columns property', function () {
        it('should render a table header cell for each column', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { columns: columns, data: PokeDex })));
            expect(component.find('[data-display="table-head"] [data-display="table-cell"]')).toHaveLength(columns.length);
        });
        describe('column.label property', function () {
            it('should use the field property as a title if a label is not provided', function () {
                var cols = [
                    {
                        field: 'Name',
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                var cell = component.find('[data-display="table-head"] [data-display="table-cell"]');
                expect(cell.text()).toEqual('Name');
            });
            it('should only render the fields specified in the `columns` attribute', function () {
                var cols = [
                    {
                        field: 'Name',
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                var cellsText = component
                    .find('[data-display="table-body"] [data-display="table-cell"]')
                    .map(function (element) { return element.text(); });
                var dataText = map(PokeDex, cols[0].field);
                expect(cellsText).toEqual(dataText);
            });
        });
        describe('column.label property', function () {
            it('should use the label property as a title if it is provided', function () {
                var cols = [
                    {
                        field: 'Name',
                        label: 'Foobarbaz',
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                var cell = component.find('[data-display="table-head"] [data-display="table-cell"]');
                expect(cell.text()).toEqual('Foobarbaz');
            });
            it('should be able to provide a JSX element as a label', function () {
                var cols = [
                    {
                        field: 'Name',
                        label: React.createElement("span", { className: "test-label" }, "Foobarbaz"),
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                expect(component.find('.test-label')).toHaveLength(1);
            });
        });
        describe('column.render property', function () {
            it('should use the render attribute to render cells if one is provided', function () {
                var cols = [
                    {
                        field: 'Name',
                        render: function (value) { return React.createElement("span", { className: "test-cell" }, value); },
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                expect(component.find('.test-cell')).toHaveLength(PokeDex.length);
            });
            it('should call the render attribute with the columns field value and the row value', function () {
                var spy = sinon.spy(function (value) { return value; });
                var field = 'Name';
                var cols = [
                    {
                        field: field,
                        render: spy,
                    },
                ];
                mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                expect(spy.callCount).toEqual(PokeDex.length);
                var calls = map(spy.getCalls(), 'args');
                var expected = map(PokeDex, function (item) { return [item[field], item]; });
                expect(calls).toEqual(expected);
            });
        });
        describe('column.sortable property', function () {
            it('should not render a button if colums are not sortable', function () {
                var cols = [
                    {
                        field: 'Name',
                    },
                    {
                        field: 'pokedex_number',
                    },
                    {
                        field: 'Category',
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                expect(component.find('[data-display="table-head"] button')).toHaveLength(0);
            });
            it('should render a button if colums are sortable', function () {
                var cols = [
                    {
                        field: 'Name',
                        sortable: true,
                    },
                    {
                        field: 'pokedex_number',
                        sortable: true,
                    },
                    {
                        field: 'Category',
                        sortable: true,
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                expect(component.find('[data-display="table-head"] button')).toHaveLength(cols.length);
            });
            it('should sort by the specified sortable column on click', function () {
                var field = 'Name';
                var cols = [
                    {
                        field: field,
                        sortable: true,
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                component.find('[data-display="table-head"] button').simulate('click');
                component.update();
                var cellsText = component
                    .find('[data-display="table-body"] [data-display="table-cell"]')
                    .map(function (element) { return element.text(); });
                var expected = map(PokeDex, field).sort();
                expect(cellsText).toEqual(expected);
            });
            it('should reverse the sort order when clicking sortable column twice', function () {
                var field = 'Name';
                var cols = [
                    {
                        field: field,
                        sortable: true,
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                component.find('[data-display="table-head"] button').simulate('click');
                component.update();
                component.find('[data-display="table-head"] button').simulate('click');
                component.update();
                var cellsText = component
                    .find('[data-display="table-body"] [data-display="table-cell"]')
                    .map(function (element) { return element.text(); });
                var expected = reverse(map(PokeDex, field).sort());
                expect(cellsText).toEqual(expected);
            });
            it('should reverse the sort order when clicking sortable column twice', function () {
                var field = 'Name';
                var cols = [
                    {
                        field: field,
                        sortable: true,
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                component.find('[data-display="table-head"] button').simulate('click');
                component.update();
                component.find('[data-display="table-head"] button').simulate('click');
                component.update();
                var cellsText = component
                    .find('[data-display="table-body"] [data-display="table-cell"]')
                    .map(function (element) { return element.text(); });
                var expected = reverse(map(PokeDex, field).sort());
                expect(cellsText).toEqual(expected);
            });
            it('should allow a custom sorting function', function () {
                var field = 'Name';
                var biasName = get(last(PokeDex), field);
                var sorter = function (a) { return (a[field] === biasName ? -1 : 1); };
                var cols = [
                    {
                        field: field,
                        sortable: sorter,
                    },
                ];
                var component = mount(React.createElement(Provider, null,
                    React.createElement(Table, { columns: cols, data: PokeDex })));
                component.find('[data-display="table-head"] button').simulate('click');
                component.update();
                var cellText = component
                    .find('[data-display="table-body"] [data-display="table-cell"]')
                    .first()
                    .text();
                expect(cellText).toEqual(biasName);
            });
        });
    });
    describe('data property', function () {
        it('should display the data fields specified in the `columns` attribute', function () {
            var cols = [
                {
                    field: 'Name',
                },
            ];
            var data = [
                {
                    id: 1,
                    Name: 'Caterpie',
                    Description: 'lorem ipsum dolor sit amet',
                },
                {
                    id: 2,
                    Name: 'Blastoise',
                    Description: 'consecteur adipiscing elit, sed do eiusmod',
                },
            ];
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { columns: cols, data: data })));
            var cellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var dataText = map(data, 'Name');
            expect(cellsText).toEqual(dataText);
        });
        it('should update the table when data is added', function () {
            var cols = [
                {
                    field: 'Name',
                },
            ];
            var data = [
                {
                    id: 1,
                    Name: 'Caterpie',
                    Description: 'lorem ipsum dolor sit amet',
                },
                {
                    id: 2,
                    Name: 'Blastoise',
                    Description: 'consecteur adipiscing elit, sed do eiusmod',
                },
            ];
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Table, __assign({}, props)))); }, { columns: cols, data: data }));
            var cellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var dataText = map(data, 'Name');
            expect(cellsText).toEqual(dataText);
            data.push({
                id: 3,
                Name: 'Squrtle',
            });
            component.setProps({ data: data });
            var updatedCellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var updatedDataText = map(data, 'Name');
            expect(updatedCellsText).toEqual(updatedDataText);
        });
        it('should update the table when data is added and a render function is used', function () {
            var cols = [
                {
                    field: 'Name',
                    render: function (value) { return React.createElement("p", null,
                        "Name: ",
                        value); },
                },
            ];
            var data = [
                {
                    id: 1,
                    Name: 'Caterpie',
                    Description: 'lorem ipsum dolor sit amet',
                },
                {
                    id: 2,
                    Name: 'Blastoise',
                    Description: 'consecteur adipiscing elit, sed do eiusmod',
                },
            ];
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Table, __assign({}, props)))); }, { columns: cols, data: data }));
            data.push({
                id: 3,
                Name: 'Squirtle',
            });
            component.setProps({ data: data });
            var cellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var dataText = map(data, function (field) { return "Name: ".concat(field.Name); });
            expect(cellsText).toEqual(dataText);
        });
        it('should not update the table when a data item changes in a mutable way', function () {
            var cols = [
                {
                    field: 'Name',
                },
            ];
            var data = [
                {
                    id: 1,
                    Name: 'Caterpie',
                    description: 'lorem ipsum dolor sit amet',
                },
                {
                    id: 2,
                    Name: 'Blastoise',
                    description: 'consecteur adipiscing elit, sed do eiusmod',
                },
            ];
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Table, __assign({}, props)))); }, { columns: cols, data: data }));
            var cellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var dataText = map(data, 'Name');
            expect(cellsText).toEqual(dataText);
            data[0].Name = 'Squirtle';
            component.setProps({ data: data });
            var updatedCellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var updatedDataText = map(data, 'Name');
            expect(updatedCellsText).not.toEqual(updatedDataText);
        });
        it('should update the table when a data item changes in an immutable way', function () {
            var cols = [
                {
                    field: 'Name',
                },
            ];
            var data = [
                {
                    id: 1,
                    Name: 'Caterpie',
                    Description: 'lorem ipsum dolor sit amet',
                },
                {
                    id: 2,
                    Name: 'Blastoise',
                    Description: 'consecteur adipiscing elit, sed do eiusmod',
                },
            ];
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Table, __assign({}, props)))); }, { columns: cols, data: data }));
            var cellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var dataText = map(data, 'Name');
            expect(cellsText).toEqual(dataText);
            data[0] = { id: 1, Name: 'Squirtle' };
            component.setProps({ data: data });
            var updatedCellsText = component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .map(function (element) { return element.text(); });
            var updatedDataText = map(data, 'Name');
            expect(updatedCellsText).toEqual(updatedDataText);
        });
    });
    describe('getRowHref property', function () {
        it('should be used to generate an href for each row', function () {
            var cols = [
                {
                    field: 'Name',
                },
            ];
            var fn = function (row) { return "https://www.pokemon.com/uk/pokedex/".concat(row.Name); };
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { getRowHref: fn, columns: cols, data: PokeDex })));
            var rowLinks = component
                .find('[data-display="table-body"] a')
                .map(function (element) { return element.prop('href'); });
            var dataLinks = PokeDex.map(fn);
            expect(rowLinks).toEqual(dataLinks);
        });
    });
    describe('onCheck property', function () {
        it('should cause a checkbox to be rendered in the header', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowKey: "pokedex_number", onCheck: noop, columns: columns, data: PokeDex })));
            expect(component.find('[data-display="table-head"] input[type="checkbox"]')).toHaveLength(1);
        });
        it('should cause a checkbox to be rendered on each row', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowKey: "pokedex_number", onCheck: noop, columns: columns, data: PokeDex })));
            expect(component.find('[data-display="table-body"] input[type="checkbox"]')).toHaveLength(PokeDex.length);
        });
        it('should show a disabled checkbox for disabled rows', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowKey: "pokedex_number", onCheck: noop, columns: columns, data: PokeDex, disabledRows: PokeDex.slice(0, 2).map(function (r) { return r.pokedex_number; }) })));
            expect(component.findWhere(function (n) {
                return (n.type() === 'input' &&
                    n.prop('type') === 'checkbox' &&
                    n.prop('disabled') === true);
            })).toHaveLength(2);
        });
        it('should select all rows except the disabled ones when clicking the header checkbox', function () {
            var disabled = 2;
            var spy = sinon.spy();
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowKey: "pokedex_number", onCheck: spy, columns: columns, data: PokeDex, disabledRows: PokeDex.slice(0, disabled).map(function (r) { return r.pokedex_number; }) })));
            component
                .find('[data-display="table-head"] input[type="checkbox"]')
                .simulate('change');
            component.update();
            var elements = component
                .find('[data-display="table-body"] input[type="checkbox"]')
                .filterWhere(function (element) { return !!element.props().checked; });
            expect(elements).toHaveLength(PokeDex.length - disabled);
            expect(spy.callCount).toEqual(1);
            expect(spy.firstCall.args[0]).toEqual(PokeDex.slice(disabled));
        });
        it('should increment or decrement the checked rows as you check/uncheck checkboxes', function () {
            var spy = sinon.spy();
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowKey: "pokedex_number", onCheck: spy, columns: columns, data: PokeDex })));
            var elements = component.find('[data-display="table-body"] input[type="checkbox"]');
            elements.first().simulate('change');
            component.update();
            expect(spy.callCount).toEqual(1);
            expect(spy.firstCall.args[0]).toEqual(PokeDex.slice(0, 1));
            elements.at(1).simulate('change');
            component.update();
            expect(spy.callCount).toEqual(2);
            expect(spy.secondCall.args[0]).toEqual(PokeDex.slice(0, 2));
            elements.at(2).simulate('change');
            component.update();
            expect(spy.callCount).toEqual(3);
            expect(spy.thirdCall.args[0]).toEqual(PokeDex.slice(0, 3));
            elements.at(2).simulate('change');
            component.update();
            expect(spy.callCount).toEqual(4);
            expect(spy.getCall(3).args[0]).toEqual(PokeDex.slice(0, 2));
            elements.at(1).simulate('change');
            component.update();
            expect(spy.callCount).toEqual(5);
            expect(spy.getCall(4).args[0]).toEqual(PokeDex.slice(0, 1));
        });
    });
    describe('onRowClick property', function () {
        it('should be called when clicking a row', function () {
            var clickIndex = 5;
            var cols = [
                {
                    field: 'Name',
                },
            ];
            var spy = sinon.spy();
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { onRowClick: spy, rowKey: "pokedex_number", columns: cols, data: PokeDex })));
            component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .first()
                .simulate('click');
            expect(spy.callCount).toEqual(1);
            expect(spy.firstCall.args[0]).toEqual(first(PokeDex));
            component
                .find('[data-display="table-body"] [data-display="table-cell"]')
                .at(clickIndex)
                .simulate('click');
            expect(spy.callCount).toEqual(2);
            expect(spy.secondCall.args[0]).toEqual(PokeDex[clickIndex]);
        });
    });
    describe('rowAnchorAttributes property', function () {
        it('should be added to each row', function () {
            var cols = [
                {
                    field: 'Name',
                },
            ];
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowAnchorAttributes: { 'data-foo': 'bar' }, columns: cols, data: PokeDex })));
            var elements = component.find('[data-display="table-body"] [data-foo="bar"]');
            expect(elements).toHaveLength(PokeDex.length);
        });
    });
    describe('tbodyPrefix property', function () {
        it('should insert an element at the top of the table body', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { tbodyPrefix: React.createElement("span", { className: "foobarbaz" }), columns: columns, data: PokeDex })));
            expect(component.find('[data-display="table-body"] span.foobarbaz')).toHaveLength(1);
        });
    });
    describe('highlightedRows property', function () {
        it('should highlight the specified row', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowKey: "pokedex_number", highlightedRows: [2], columns: [{ field: 'Name' }], data: PokeDex })));
            var match = component.find('[data-highlight=true]');
            expect(match).toHaveLength(1);
            expect(match.text()).toEqual('Ivysaur');
        });
        it('should be able to highlight multiple rows', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { rowKey: "pokedex_number", highlightedRows: [3, 5], columns: [{ field: 'Name' }], data: PokeDex })));
            var match = component.find('[data-highlight=true]');
            expect(match).toHaveLength(2);
            expect(match.map(function (e) { return e.text(); })).toEqual(['Venusaur', 'Charmeleon']);
        });
    });
    describe('itemsPerPage property', function () {
        it('should limit the number of items shown at one time', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { columns: [{ field: 'Name' }], data: PokeDex, usePager: true, pagination: { itemsPerPage: 3 } })));
            var match = component.find('[data-display="table-body"] [data-display="table-row"]');
            expect(match).toHaveLength(3);
            var result = PokeDex.slice(0, 3).map(function (p) { return p.Name; });
            expect(match.map(function (e) { return e.text(); })).toEqual(result);
        });
    });
    describe('usePager property', function () {
        it('should display a pager', function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { columns: [{ field: 'Name' }], data: PokeDex, usePager: true })));
            expect(component.find(Pager)).toHaveLength(1);
        });
        it("shouldn't display a pager if there are no items", function () {
            var component = mount(React.createElement(Provider, null,
                React.createElement(Table, { columns: [{ field: 'Name' }], data: [], usePager: true })));
            expect(component.find(Pager)).toHaveLength(0);
        });
        it('should reset the page if the data length becomes less than the current pager value', function () {
            var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
                React.createElement(Table, __assign({}, props)))); }, {
                columns: [{ field: 'Name' }],
                data: PokeDex,
                pagination: { itemsPerPage: 2 },
                usePager: true,
            }));
            var table = component.find(TableBase).instance();
            expect(table.state.page).toEqual(0);
            table.incrementPage();
            table.incrementPage();
            expect(table.state.page).toEqual(2);
            component.setProps({ data: PokeDex.slice(0, 2) });
            expect(table.state.page).toEqual(0);
        });
    });
});
describe('getRowClass property', function () {
    it('should add a class to every row', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", getRowClass: getRowClass, columns: columns, data: PokeDex })));
        var elements = component.find('[data-display="table-body"] [data-display="table-row"]');
        expect(elements).toHaveLength(PokeDex.length);
        elements.forEach(function (node) {
            expect(node.hasClass('pokemon')).toEqual(true);
        });
    });
    it('should not add any classes if not present', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", columns: columns, data: PokeDex })));
        var elements = component.find('[data-display="table-body"] [data-display="table-row"]');
        expect(elements).toHaveLength(PokeDex.length);
        elements.forEach(function (node) {
            expect(node.hasClass('pokemon')).toEqual(false);
        });
    });
});
describe('control selected rows', function () {
    it('should select the first two rows', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", columns: columns, data: PokeDex, checkedItems: PokeDex.slice(0, 2) })));
        var table = component.find(TableBase).instance();
        expect(table.state.checkedItems.length).toEqual(2);
        expect(table.state.checkedState).toEqual('some');
    });
});
describe('manually select rows', function () {
    it('should select the first two rows', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", columns: columns, data: PokeDex })));
        var table = component.find(TableBase).instance();
        expect(table.state.checkedItems.length).toEqual(0);
        table.setRowSelection(PokeDex.slice(0, 2));
        expect(table.state.checkedItems.length).toEqual(2);
        expect(table.state.checkedState).toEqual('some');
    });
    it('should select all rows', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", columns: columns, data: PokeDex })));
        var table = component.find(TableBase).instance();
        expect(table.state.checkedItems.length).toEqual(0);
        table.setRowSelection(PokeDex);
        expect(table.state.checkedItems.length).toEqual(PokeDex.length);
        expect(table.state.checkedState).toEqual('all');
    });
    it('should clear all rows', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", columns: columns, data: PokeDex })));
        var table = component.find(TableBase).instance();
        expect(table.state.checkedItems.length).toEqual(0);
        table.setRowSelection(PokeDex);
        expect(table.state.checkedItems.length).toEqual(PokeDex.length);
        expect(table.state.checkedState).toEqual('all');
        table.setRowSelection([]);
        expect(table.state.checkedItems.length).toEqual(0);
        expect(table.state.checkedState).toEqual('none');
    });
    it('should clear selection if data is set to null after instantiation', function () {
        var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
            React.createElement(Table, __assign({}, props)))); }, { columns: columns, rowKey: 'pokedex_number', data: PokeDex }));
        var table = component.find(TableBase).instance();
        expect(table.state.checkedItems.length).toEqual(0);
        table.setRowSelection(PokeDex);
        expect(table.state.checkedItems.length).toEqual(PokeDex.length);
        component.setProps({ data: null });
        table.setRowSelection([]);
        expect(table.state.checkedItems.length).toEqual(0);
        expect(table.state.checkedState).toEqual('none');
    });
});
describe('sorting', function () {
    it('should invoke the sort callback when sort header is clicked', function () {
        var spy = sinon.spy();
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", onSort: spy, columns: columns, data: PokeDex })));
        var elements = component.find('[data-display="table-head"] [data-display="table-cell"] button');
        elements.first().simulate('click');
        component.update();
        expect(spy.callCount).toEqual(1);
        expect(spy.firstCall.args[0]).toEqual({ field: 'Name', reverse: false });
    });
    it('should set the sorted column based on the passed sort property', function () {
        var defaultSort = { field: 'Name', reverse: true };
        var updatedSort = { field: 'pokedex_number', reverse: false };
        var component = mount(React.createElement(function (props) { return (React.createElement(Provider, null,
            React.createElement(Table, __assign({}, props)))); }, {
            rowKey: 'pokedex_number',
            sort: defaultSort,
            columns: columns,
            data: PokeDex,
        }));
        var table = component.find(TableBase).instance();
        expect(table.state.sort).toEqual(defaultSort);
        component.setProps({ sort: updatedSort });
        expect(table.state.sort).toEqual(updatedSort);
    });
});
describe('custom columns', function () {
    it('should render the Table component with a cog', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", columns: columns, data: PokeDex, enableCustomColumns: true })));
        var cog = component.find(TableColumnSelectorSizer);
        expect(cog).toBeDefined();
    });
    it('should render only selected columns', function () {
        var component = mount(React.createElement(Provider, null,
            React.createElement(Table, { rowKey: "pokedex_number", columns: columns, data: PokeDex, enableCustomColumns: true })));
        var cols = component.find('[data-display="table-head"] [data-display="table-cell"]');
        var numberOfColumnsToShow = columns.filter(function (c) { return c.selected; }).length;
        expect(cols).toHaveLength(numberOfColumnsToShow);
    });
});
